package com.fjhx.mes.service.work.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.item.entity.product.po.ProductInfo;
import com.fjhx.item.service.product.ProductInfoService;
import com.fjhx.mes.entity.production.po.ProductionPlan;
import com.fjhx.mes.entity.work.dto.WorkOrderDto;
import com.fjhx.mes.entity.work.dto.WorkOrderSelectDto;
import com.fjhx.mes.entity.work.po.WorkOrder;
import com.fjhx.mes.entity.work.vo.WorkOrderVo;
import com.fjhx.mes.mapper.work.WorkOrderMapper;
import com.fjhx.mes.service.production.ProductionPlanService;
import com.fjhx.mes.service.work.WorkOrderService;
import com.fjhx.mes.utils.code.CodeEnum;
import com.fjhx.sale.entity.contract.po.ContractProduct;
import com.fjhx.sale.service.contract.ContractProductService;
import com.fjhx.sale.service.contract.impl.ContractProductServiceImpl;
import com.obs.services.internal.ServiceException;
import com.ruoyi.common.utils.wrapper.IWrapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


/**
 * <p>
 * 工单 服务实现类
 * </p>
 *
 * @author
 * @since 2023-03-29
 */
@Service
public class WorkOrderServiceImpl extends ServiceImpl<WorkOrderMapper, WorkOrder> implements WorkOrderService {

    @Autowired
    ProductInfoService productInfoService;
    @Autowired
    ProductionPlanService productionPlanService;

    @Autowired
    private ContractProductService contractProductService;

    @Override
    public Page<WorkOrderVo> getPage(WorkOrderSelectDto dto) {
        IWrapper<WorkOrder> wrapper = getWrapper();
        wrapper.orderByDesc("wo", WorkOrder::getId);
        wrapper.like("wo", WorkOrder::getCode, dto.getKeyword());
        wrapper.or();
        wrapper.like("wo", WorkOrder::getQuantity, dto.getKeyword());
        Page<WorkOrderVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        List<WorkOrderVo> records = page.getRecords();
        //赋值产品名
        List<Long> productIds = records.stream().map(WorkOrder::getProductId).collect(Collectors.toList());
        if (ObjectUtil.isNotEmpty(productIds)) {
            //赋值产品名称
            productInfoService.attributeAssign(records, WorkOrderVo::getProductId, (item, productInfo) -> {
                item.setProductName(productInfo.getName());
            });
        }
        //赋值待排程数量
        List<Long> workOrderIds = records.stream().map(WorkOrder::getId).collect(Collectors.toList());
        if (ObjectUtil.isNotEmpty(workOrderIds)) {
            List<ProductionPlan> productionPlanList = productionPlanService.list(q -> q.in(ProductionPlan::getWorkOrderId, workOrderIds));
            if (ObjectUtil.isNotEmpty(productionPlanList)) {
                //根据工单id分组
                Map<Long, List<ProductionPlan>> productionPlanMap = productionPlanList.stream().collect(Collectors.groupingBy(ProductionPlan::getWorkOrderId));
                for (WorkOrderVo workOrder : records) {
                    List<ProductionPlan> productionPlanList1 = productionPlanMap.get(workOrder.getId());
                    BigDecimal count = BigDecimal.ZERO;
                    if (ObjectUtil.isNotEmpty(productionPlanList1)) {
                        count = productionPlanList1.stream().map(ProductionPlan::getQuantity)
                                .reduce(BigDecimal.ZERO, BigDecimal::add);
                    }
                    workOrder.setRemainingQuantity(workOrder.getQuantity().subtract(count));
                }
            }
        }
        return page;
    }

    @Override
    public WorkOrderVo detail(Long id) {
        WorkOrder WorkOrder = this.getById(id);
        WorkOrderVo result = BeanUtil.toBean(WorkOrder, WorkOrderVo.class);
        List<ProductionPlan> list = productionPlanService.list(q -> q.eq(ProductionPlan::getWorkOrderId, id));
        result.setProductionPlans(list);
        ProductInfo productInfo = productInfoService.getById(result.getProductId());
        if (ObjectUtil.isNotEmpty(productInfo)) {
            result.setProductName(productInfo.getName());
        }
        return result;
    }

    @Override
    public void add(WorkOrderDto workOrderDto) {
        //生成工单编号,以及设置默认状态
        workOrderDto.setCode(CodeEnum.WORK_ORDER.getCode());
        workOrderDto.setStatus(0);
        this.save(workOrderDto);
    }

    /**
     * 批量新增
     * @param workOrder
     */
    @Override
    @DSTransactional
    public void addBatch(List<WorkOrder> workOrder) {
        if(CollectionUtils.isEmpty(workOrder)){
            throw new ServiceException("提交不能为空");
        }
        List<ContractProduct> upContractProduct = new ArrayList<>();
        for(WorkOrder w:workOrder){
            //生成工单编号,以及设置默认状态
            w.setCode(CodeEnum.WORK_ORDER.getCode());
            w.setStatus(0);
            if(ObjectUtil.isNotEmpty(w.getSourceId())&&w.getSource()==1){//外销合同生产数据
                ContractProduct contractProduct = contractProductService.getById(w.getSourceId());
                if(ObjectUtil.isNotEmpty(contractProduct)){
                    throw new ServiceException("该合同产品不存在,请检查");
                }
                BigDecimal expendQuantity = contractProduct.getExpendQuantity().subtract(w.getQuantity());
                if(expendQuantity.compareTo(BigDecimal.ZERO)< 1){//小于0不让继续执行
                    throw new ServiceException("生产数量不得大于销售合同数量");
                }
                contractProduct.setExpendQuantity(expendQuantity);
                upContractProduct.add(contractProduct);
            }
        }
        this.saveBatch(workOrder);
        if(CollectionUtils.isNotEmpty(upContractProduct)){//扣减销售合同数量
            contractProductService.updateBatchById(upContractProduct);
        }
    }

    @Override
    public void edit(WorkOrderDto workOrderDto) {
        this.updateById(workOrderDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

}
